﻿
using System;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using Rosieks.Web.Flow.Html;
using Rosieks.Web.Flow.Metadata;
using Rosieks.Web.Flow.Tests.Helpers;

namespace Rosieks.Web.Flow.Tests.Html
{
    [TestClass]
    public class HtmlExtensionsTest
    {
        [TestMethod]
        public void FlowButton_RegularTransition()
        {
            var viewContext = new ViewContext();
            var helper = new HtmlHelper(viewContext, HtmlHelperTest.GetViewDataContainer(null));
            var httpContext = ContextHelper.CreateHttpContext().WithWebflow(new ViewState { Transitions = new[] { new Transition("next", null) } }, null);

            viewContext.HttpContext = httpContext.Object;

            var html = helper.FlowButton("Next >>", "next", new { @class = "btn", data_bind = "text: value" });

            Assert.AreEqual(@"<input class=""btn"" data-bind=""text: value"" data-flow-default=""false"" name=""ActionName.next"" type=""submit"" value=""Next >>"" />" +
                            @"<input name=""ActionName"" type=""hidden"" value=""next"" />",
                            html.ToHtmlString());
        }

        [TestMethod]
        public void FlowButton_TransitionWithoutValidation_RenderWithIgnoreRules()
        {
            var viewContext = new ViewContext();
            var helper = new HtmlHelper(viewContext, HtmlHelperTest.GetViewDataContainer(null));
            var httpContext = ContextHelper.CreateHttpContext().WithWebflow(new ViewState { Transitions = new[] { new Transition("next", null) { RequireValidModel = false } } }, null);

            viewContext.HttpContext = httpContext.Object;

            var html = helper.FlowButton("Next >>", "next", new { @class = "btn" });

            Assert.AreEqual(@"<input class=""btn"" data-flow-default=""false"" data-val-ignore-rules=""true"" name=""ActionName.next"" type=""submit"" value=""Next >>"" />" +
                            @"<input name=""ActionName"" type=""hidden"" value=""next"" />",
                            html.ToHtmlString());
        }

        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void FlowButton_WithUnknownTransition_ThrowArgumentException()
        {
            var viewContext = new ViewContext();
            var helper = new HtmlHelper(viewContext, HtmlHelperTest.GetViewDataContainer(null));
            var httpContext = ContextHelper.CreateHttpContext().WithWebflow(new ViewState { Transitions = new[] { new Transition("next", null) } }, null);

            viewContext.HttpContext = httpContext.Object;

            var html = helper.FlowButton("Next >>", "back", new { @class = "btn", data_bind = "text: value" });
        }

        [TestMethod]
        public void FlowImageButtonTest_WithContentImageUrl()
        {
            var httpResponse = new Mock<HttpResponseBase>();
            var httpRequest = new Mock<HttpRequestBase>();
            var httpContext = ContextHelper.CreateHttpContext().WithWebflow(new ViewState { Transitions = new[] { new Transition("next", null) } }, null);
            var requestContext = new RequestContext();
            var viewContext = new ViewContext();
            var helper = new HtmlHelper(viewContext, HtmlHelperTest.GetViewDataContainer(null));

            httpRequest.Setup(r => r.ApplicationPath).Returns("/");
            httpContext.Setup(c => c.Request).Returns(httpRequest.Object);
            httpContext.Setup(c => c.Response).Returns(httpResponse.Object);
            requestContext.HttpContext = httpContext.Object;
            viewContext.RequestContext = requestContext;

            var html = helper.FlowImageButton("/picture.gif", "Next >>", "next", new { @class = "btn" });

            Assert.AreEqual(@"<input class=""btn"" data-flow-default=""false"" name=""ActionName.next"" src=""/picture.gif"" type=""image"" value=""Next >>"" />" +
                            @"<input name=""ActionName"" type=""hidden"" value=""next"" />",
                            html.ToHtmlString());
        }

        [TestMethod]
        public void FlowLinkTest()
        {
            var currentState = new Mock<State>();
            var httpContext = ContextHelper.CreateHttpContext().WithWebflow(currentState.Object, null).Object;
            var requestContext = new RequestContext();
            var viewContext = new ViewContext();
            var helper = new HtmlHelper(viewContext, HtmlHelperTest.GetViewDataContainer(null));

            currentState.Object.RouteData = new RouteData { RouteName = "haha", Values = new RouteValueDictionary(new { controller = "Flow", action = "OnState" }) };
            currentState.Object.Transitions = new Transition[] { new Transition("prev", null) };
            requestContext.HttpContext = httpContext;
            viewContext.RequestContext = requestContext;

            var html = helper.FlowLink("Wstecz", "prev", new { @class = "link" });

            Assert.AreEqual(@"<a class=""link"" href=""Flow/OnState?actionName=prev"">Wstecz</a>", html.ToHtmlString());
        }
    }
}
