﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using Rosieks.Web.Flow.Metadata;
using Rosieks.Web.Flow.Tests.Helpers;

namespace Rosieks.Web.Flow.Tests
{
    [TestClass]
    public class WebflowActionAttributeTest
    {
        [TestMethod]
        [ExpectedException(typeof(InvalidOperationException))]
        public void OnWebflowActionExecuting_NameNotAssigned()
        {
            var actionExecutingContext = new ActionExecutingContext();

            var webflowAction = new WebflowActionAttribute();

            webflowAction.OnActionExecuting(actionExecutingContext);
        }

        [TestMethod]
        public void OnWebflowActionExecuting()
        {
            var wc = new WebflowContext() { CurrentState = new Mock<State>().Object };
            var items = new Dictionary<object, object>();

            var webflowAction = new WebflowActionAttribute();
            var actionContext = new ActionExecutingContext();
            var httpContext = new Mock<HttpContextBase>();
            actionContext.HttpContext = httpContext.Object;
            httpContext.Setup(c => c.Items).Returns(items);

            webflowAction.Name = "myWebflow";
            webflowAction.OnActionExecuting(actionContext);

            Assert.AreEqual(wc, items["__CurrentWebflow__"]);
        }

        [TestMethod]
        public void OnWebflowActionExecuting_UrlDoesntMatchCurrentState_Redirect()
        {
            var checkStatePolicy = new Mock<ICheckStatePolicy>();
            var webflow = new WebflowModel("Lorem ipsum") { CheckStatePolicy = checkStatePolicy.Object };
            var manager = new Mock<IWebflowManager>();
            var httpContext = ContextHelper.CreateHttpContext().WithWebflow(new Mock<State>().Object, webflow);
            var controllerContext = new ControllerContext(httpContext.Object, new System.Web.Routing.RouteData(), new FakeController());
            var actionContext = new ActionExecutingContext();
            var routeData = new RouteData() { RouteName = "Wizard", Values = new RouteValueDictionary(new { step = 1 }) };

            checkStatePolicy.Setup(p => p.IsCurrentStateValid(It.IsAny<WebflowContext>(), It.IsAny<RequestContext>(), out routeData));
            actionContext.Controller = controllerContext.Controller;
            actionContext.HttpContext = httpContext.Object;

            var webflowAction = new WebflowActionAttribute();
            webflowAction.Manager = manager.Object;
            webflowAction.OnActionExecuting(actionContext);

            var result = actionContext.Result as RedirectToRouteResult;
            Assert.IsNotNull(result);
            Assert.AreEqual("Wizard", result.RouteName);
            Assert.AreEqual(1, result.RouteValues["step"]);
        }

        [TestMethod]
        public void OnWebflowActionExecuting_NameAsActionParam()
        {
            var wc = new WebflowContext() { CurrentState = new Mock<State>().Object };
            var items = new Dictionary<object, object>();
            var actionParams = new Dictionary<string, object>() { { "flowName", "myWebflow" } };

            var webflowAction = new WebflowActionAttribute();
            var actionContext = new ActionExecutingContext();
            var httpContext = new Mock<HttpContextBase>();
            httpContext.Setup(c => c.Items).Returns(items);
            actionContext.HttpContext = httpContext.Object;
            actionContext.ActionParameters = actionParams;

            webflowAction.NameByParam = "flowName";
            webflowAction.OnActionExecuting(actionContext);

            Assert.AreEqual(wc, items["__CurrentWebflow__"]);
        }

        [TestMethod]
        public void OnWebflowActionExecuted()
        {
            var state2 = new ViewState { Name = "State2" };
            var transition = new Transition { On = "Next", To = state2 };
            var state1 = new ViewState
                         {
                             Name = "State1",
                             Transitions = new[]
                                           {
                                               transition,
                                           },
                         };

            var wc = new WebflowContext { CurrentState = state1, Webflow = new WebflowModel("myWebflow") };
            var result = new WebflowResult("Next");
            var items = new Dictionary<object, object> { { "__CurrentWebflow__", wc } };

            var webflowAction = new WebflowActionAttribute();
            var actionContext = new ActionExecutedContext();
            var httpContext = new Mock<HttpContextBase>();

            httpContext.Setup(c => c.Items).Returns(items);
            actionContext.HttpContext = httpContext.Object;
            actionContext.Result = result;

            Assert.AreEqual("State1", wc.CurrentState.Name);

            webflowAction.Name = "myWebflow";
            webflowAction.OnActionExecuted(actionContext);

            Assert.AreEqual("State2", wc.CurrentState.Name);
            Assert.AreEqual(transition, wc.CurrentElement);
        }
    }
}
