﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Web.Mvc;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using Rosieks.Web.Flow.Metadata;
using Rosieks.Web.Flow.Tests.Helpers;

namespace Rosieks.Web.Flow.Tests
{
    [TestClass]
    public class WebflowManagerTest
    {
        [TestMethod]
        public void Load_ContextExistInRequest_LoadedAndSetup()
        {
            var httpContext = ContextHelper.CreateHttpContext();
            var webflowModel = new Mock<WebflowModel>();
            var viewState = new ViewState("view");
            webflowModel.Object.States = new[] { viewState };

            // Initialize dependency
            var provider = new Mock<IWebflowContextProvider>();
            var catalog = new Mock<IWebflowCatalog>();
            var webflowContext = new WebflowContext
            {
                CurrentStateName = "view",
            };

            provider.Setup(p => p.GetContext(It.IsAny<ControllerContext>(), "webflow1")).Returns(webflowContext);
            catalog.Setup(c => c.Load("webflow1")).Returns(webflowModel.Object);

            WebflowContextProviders.Providers.Clear();
            WebflowContextProviders.Providers.Add("TempData", provider.Object);

            // Initialize target
            var target = new WebflowManager(catalog.Object);

            // Initialize parameters
            ControllerContext controllerContext = new ControllerContext();
            controllerContext.HttpContext = httpContext.Object;

            // Test
            target.Load(controllerContext, "webflow1");
            var actualWebflowContext = httpContext.Object.Items["__CurrentWebflow__"] as WebflowContext;

            // Verify
            Assert.AreEqual(webflowContext, actualWebflowContext);
            Assert.AreEqual(viewState, actualWebflowContext.CurrentState);
        }

        [TestMethod]
        public void Load_ContextDoesNotExistInRequest_CreateNewAndSetup()
        {
            var httpContext = ContextHelper.CreateHttpContext();
            var webflowModel = new Mock<WebflowModel>();
            var viewState = new ViewState("view");
            webflowModel.Object.States = new[] { viewState };
            webflowModel.Object.FirstState = viewState;

            // Initialize dependency
            var provider = new Mock<IWebflowContextProvider>();
            var catalog = new Mock<IWebflowCatalog>();

            catalog.Setup(c => c.Load("webflow1")).Returns(webflowModel.Object);

            WebflowContextProviders.Providers.Clear();
            WebflowContextProviders.Providers.Add("TempData", provider.Object);

            // Initialize target
            var target = new WebflowManager(catalog.Object);

            // Initialize parameters
            ControllerContext controllerContext = new ControllerContext();
            controllerContext.HttpContext = httpContext.Object;

            // Test
            target.Load(controllerContext, "webflow1");
            var actualWebflowContext = httpContext.Object.Items["__CurrentWebflow__"] as WebflowContext;

            // Verify
            Assert.AreEqual(viewState, actualWebflowContext.CurrentState);
        }

        [TestMethod]
        public void Save_ContextInProgress_Saved()
        {
            WebflowContext webflowContext = null;
            var httpContext = ContextHelper.CreateHttpContext().WithWebflow(new ViewState(), null, ref webflowContext);

            // Initialize dependencies
            var provider = new Mock<IWebflowContextProvider>();
            var catalog = new Mock<IWebflowCatalog>();

            WebflowContextProviders.Providers.Clear();
            WebflowContextProviders.Providers.Add("TempData", provider.Object);

            // Initialize target
            var target = new WebflowManager(catalog.Object);

            // Initialize parameters
            ControllerContext controllerContext = new ControllerContext();
            controllerContext.HttpContext = httpContext.Object;

            // Test
            target.Save(controllerContext);

            // Verify
            provider.Verify(p => p.SetContext(controllerContext, webflowContext));
        }

        [TestMethod]
        public void Save_ContextIsComplete_CompleteFlowAndRemoveContext()
        {
            
            WebflowContext webflowContext = null;
            var webflowModel = new Mock<WebflowModel>();
            var httpContext = ContextHelper.CreateHttpContext().WithWebflow(null, webflowModel.Object, ref webflowContext);

            // Initialize dependencies
            var provider = new Mock<IWebflowContextProvider>();
            var catalog = new Mock<IWebflowCatalog>();

            WebflowContextProviders.Providers.Clear();
            WebflowContextProviders.Providers.Add("TempData", provider.Object);

            // Initialize target
            var target = new WebflowManager(catalog.Object);

            // Initialize parameters
            ControllerContext controllerContext = new ControllerContext();
            controllerContext.HttpContext = httpContext.Object;

            // Test
            target.Save(controllerContext);

            // Verify
            webflowModel.Verify(p => p.OnFinish(webflowContext));
            provider.Verify(p => p.DeleteContext(controllerContext, webflowContext));
        }
    }
}
